"use strict";
var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const fp = require("fastify-plugin");
const api = require("zod-openapi/api");
const fastJsonStringify = require("fast-json-stringify");
const zodOpenapi = require("zod-openapi");
const error = require("@fastify/error");
const swagger = require("@fastify/swagger");
const _interopDefaultCompat = (e) => e && typeof e === "object" && "default" in e ? e : { default: e };
const fp__default = /* @__PURE__ */ _interopDefaultCompat(fp);
const fastJsonStringify__default = /* @__PURE__ */ _interopDefaultCompat(fastJsonStringify);
const FASTIFY_ZOD_OPENAPI_CONFIG = Symbol("fastify-zod-openapi-config");
const fastifyZodOpenApi = async (fastify, opts) => {
  const registry = api.createRegistry(opts.components);
  fastify.addHook("onRoute", ({ schema }) => {
    if (!schema || schema.hide) {
      return;
    }
    schema[FASTIFY_ZOD_OPENAPI_CONFIG] ?? (schema[FASTIFY_ZOD_OPENAPI_CONFIG] = {
      registry,
      documentOpts: opts.documentOpts,
      fastifyComponents: {
        responses: /* @__PURE__ */ new Map(),
        requestBodies: /* @__PURE__ */ new Map()
      }
    });
  });
};
const fastifyZodOpenApiPlugin = fp__default.default(fastifyZodOpenApi, {
  name: "fastify-zod-openapi"
});
class RequestValidationError extends Error {
  constructor(keyword, instancePath, schemaPath, message, params) {
    super(message, {
      cause: params.issue
    });
    __publicField(this, "cause");
    this.keyword = keyword;
    this.instancePath = instancePath;
    this.schemaPath = schemaPath;
    this.message = message;
    this.params = params;
  }
}
class ResponseSerializationError extends error.createError(
  "FST_ERR_RESPONSE_SERIALIZATION",
  "Response does not match the schema",
  500
) {
  constructor(method, url, options) {
    super();
    __publicField(this, "cause");
    this.method = method;
    this.url = url;
    this.cause = options.cause;
  }
}
const createSerializerCompiler = (opts) => (routeSchema) => {
  const { schema, url, method } = routeSchema;
  if (!api.isAnyZodType(schema)) {
    return (opts == null ? void 0 : opts.fallbackSerializer) ? opts.fallbackSerializer(routeSchema) : fastJsonStringify__default.default(schema);
  }
  let stringify = opts == null ? void 0 : opts.stringify;
  if (!stringify) {
    const { schema: jsonSchema, components } = zodOpenapi.createSchema(schema, {
      registry: api.createRegistry({
        schemas: opts == null ? void 0 : opts.components
      }),
      schemaRefPath: "#/definitions/"
    });
    const maybeDefinitions = components ? {
      definitions: components
    } : void 0;
    stringify = fastJsonStringify__default.default({
      ...jsonSchema,
      ...maybeDefinitions
    });
  }
  return (value) => {
    const result = schema.safeParse(value);
    if (!result.success) {
      throw new ResponseSerializationError(method, url, {
        cause: result.error
      });
    }
    return stringify(result.data);
  };
};
const serializerCompiler = createSerializerCompiler();
const createParams = (parameters, type, registry, path) => {
  const params = {};
  for (const [key, value] of Object.entries(parameters._zod.def.shape)) {
    const parameter = registry.addParameter(value, path, {
      location: {
        in: type,
        name: key
      }
    });
    if ("$ref" in parameter || !parameter.schema) {
      throw new Error("References not supported");
    }
    const { in: inLocation, name, schema, ...rest } = parameter;
    params[key] = rest;
  }
  return params;
};
const createResponse = (response, contentTypes, registry, responseComponents, path) => {
  if (typeof response !== "object" || response == null) {
    return response;
  }
  const responsesObject = {};
  for (const [key, value] of Object.entries(response)) {
    const unknownValue = value;
    if (api.isAnyZodType(unknownValue)) {
      if (!(contentTypes == null ? void 0 : contentTypes.length)) {
        responsesObject[key] = registry.addSchema(
          unknownValue,
          [...path, key, "content", "application/json", "schema"],
          {
            io: "output",
            source: {
              type: "mediaType"
            }
          }
        );
        continue;
      }
      const contentSchemas = contentTypes.map(
        (contentType) => registry.addSchema(
          unknownValue,
          [...path, key, "content", contentType, "schema"],
          {
            io: "output",
            source: {
              type: "mediaType"
            }
          }
        )
      );
      responsesObject[key] = contentSchemas[0];
      continue;
    }
    const responsePath = [...path, key];
    const responseObject = registry.addResponse(
      unknownValue,
      responsePath
    );
    if ("$ref" in responseObject && typeof responseObject.$ref === "string") {
      responseComponents.set(responseObject.$ref, {
        referenceObject: responseObject,
        path: responsePath
      });
    }
    responsesObject[key] = responseObject;
  }
  return responsesObject;
};
const createBody = (body, contentTypes, routePath, registry, bodyComponents) => {
  if (!body) {
    return void 0;
  }
  if (api.isAnyZodType(body)) {
    if (!(contentTypes == null ? void 0 : contentTypes.length)) {
      const bodySchema = registry.addSchema(
        body,
        [...routePath, "requestBody", "content", "application/json", "schema"],
        {
          io: "input",
          source: {
            type: "mediaType"
          }
        }
      );
      bodySchema["x-fastify-zod-openapi-optional"] = body._zod.optin === "optional";
      return bodySchema;
    }
    const bodySchemas = contentTypes.map((contentType) => {
      const schema = registry.addSchema(
        body,
        [...routePath, "requestBody", "content", contentType, "schema"],
        {
          io: "input",
          source: {
            type: "mediaType"
          }
        }
      );
      schema["x-fastify-zod-openapi-optional"] = body._zod.optin === "optional";
      return schema;
    });
    return bodySchemas[0];
  }
  const requestBodyPath = [...routePath, "requestBody"];
  const requestBodyObject = registry.addRequestBody(
    body,
    requestBodyPath
  );
  if ("$ref" in requestBodyObject && typeof requestBodyObject.$ref === "string") {
    bodyComponents.set(requestBodyObject.$ref, {
      referenceObject: requestBodyObject,
      path: requestBodyPath
    });
  }
  return requestBodyObject;
};
const fastifyZodOpenApiTransform = ({
  schema,
  url,
  ...opts
}) => {
  var _a;
  if (!schema || schema.hide) {
    return {
      schema,
      url
    };
  }
  const { response, headers, querystring, body, params, ...rest } = schema;
  if (!("openapiObject" in opts)) {
    throw new Error("openapiObject was not found in the options");
  }
  const config = schema[FASTIFY_ZOD_OPENAPI_CONFIG];
  if (!config) {
    throw new Error("Please register the fastify-zod-openapi plugin");
  }
  const { registry } = config;
  (_a = opts.openapiObject)[FASTIFY_ZOD_OPENAPI_CONFIG] ?? (_a[FASTIFY_ZOD_OPENAPI_CONFIG] = config);
  const fastifySchema = rest;
  const routeMethod = opts.route.method.toLowerCase();
  const routePath = ["paths", swagger.formatParamUrl(url), routeMethod];
  const maybeBody = createBody(
    body,
    rest.consumes,
    routePath,
    registry,
    config.fastifyComponents.requestBodies
  );
  if (maybeBody) {
    fastifySchema.body = maybeBody;
  }
  const maybeResponse = createResponse(
    response,
    rest.produces,
    registry,
    config.fastifyComponents.responses,
    [...routePath, "responses"]
  );
  if (maybeResponse) {
    fastifySchema.response = maybeResponse;
  }
  const parameterPath = [...routePath, "parameters"];
  if (api.isAnyZodType(querystring)) {
    const queryStringSchema = api.unwrapZodObject(querystring, "input", [
      ...parameterPath,
      "query"
    ]);
    fastifySchema.querystring = createParams(
      queryStringSchema,
      "query",
      registry,
      parameterPath
    );
  }
  if (api.isAnyZodType(params)) {
    const paramsSchema = api.unwrapZodObject(params, "input", [
      ...parameterPath,
      "path"
    ]);
    fastifySchema.params = createParams(
      paramsSchema,
      "path",
      registry,
      parameterPath
    );
  }
  if (api.isAnyZodType(headers)) {
    const headersSchema = api.unwrapZodObject(headers, "input", [
      ...parameterPath,
      "header"
    ]);
    fastifySchema.headers = createParams(
      headersSchema,
      "header",
      registry,
      parameterPath
    );
  }
  return {
    schema: fastifySchema,
    url
  };
};
const resolveSchemaComponent = (object, registry) => {
  if (typeof object.$ref === "string") {
    const id = object.$ref.replace("#/components/schemas/", "");
    return registry.components.schemas.ids.get(id);
  }
  return object;
};
const traverseObject = (openapiObject, source, schemaObject, registry) => {
  var _a, _b, _c, _d;
  let index = 0;
  let current = openapiObject;
  while (index < source.path.length) {
    const key = source.path[index++];
    if (typeof current !== "object" || current === null || !(key in current)) {
      return void 0;
    }
    current = current[key];
    if (typeof current === "object" && current !== null && "$ref" in current && typeof current.$ref === "string") {
      return current;
    }
    if (key === "requestBody" && typeof current === "object") {
      const requestBody = current;
      if (source.type === "requestBody") {
        delete requestBody.content;
        delete requestBody.required;
        delete requestBody.description;
        Object.assign(requestBody, schemaObject);
        return requestBody;
      }
      const contentType = (_a = source.path) == null ? void 0 : _a[index + 1];
      if (!contentType) {
        return void 0;
      }
      const schema = (_c = (_b = requestBody.content) == null ? void 0 : _b[contentType]) == null ? void 0 : _c.schema;
      if (!schema) {
        return void 0;
      }
      if ("$ref" in schema && schema.$ref) {
        return schema;
      }
      const resolved = resolveSchemaComponent(schemaObject, registry);
      const description = schemaObject.description ?? resolved.description;
      if (description) {
        requestBody.description = description;
      }
      Object.assign(schema, schemaObject);
      if (schema["x-fastify-zod-openapi-optional"] === false) {
        requestBody.required = true;
        delete schema["x-fastify-zod-openapi-optional"];
      }
      return schema;
    }
    if (key === "parameters" && typeof current === "object" && Array.isArray(current) && source.type === "parameter") {
      const parameter = current.find(
        (param) => param.name === source.location.name && param.in === source.location.in
      );
      if (parameter == null ? void 0 : parameter.schema) {
        return Object.assign(
          parameter.schema,
          schemaObject
        );
      }
      return void 0;
    }
    if (key === "responses" && typeof current === "object" && source.type === "response") {
      const responses = current;
      const statusCode = (_d = source.path) == null ? void 0 : _d[index];
      if (!statusCode) {
        return void 0;
      }
      responses[statusCode] = schemaObject;
      return responses[statusCode];
    }
  }
  if (typeof current === "object" && current !== null && Object.keys(current).length !== 0) {
    return current;
  }
  return Object.assign(current, schemaObject);
};
const combineComponents = (existingComponents, newComponents) => {
  const allComponents = [
    "schemas",
    "parameters",
    "responses",
    "requestBodies",
    "securitySchemes",
    "examples",
    "links",
    "headers",
    "callbacks"
  ];
  for (const key of allComponents) {
    if (existingComponents[key] || newComponents[key]) {
      existingComponents[key] = {
        ...existingComponents[key],
        ...newComponents[key]
      };
    }
  }
  return existingComponents;
};
const fastifyZodOpenApiTransformObject = (opts) => {
  if ("swaggerObject" in opts) {
    return opts.swaggerObject;
  }
  const config = opts.openapiObject[FASTIFY_ZOD_OPENAPI_CONFIG];
  if (!config) {
    return opts.openapiObject;
  }
  const components = api.createComponents(
    config.registry,
    config.documentOpts ?? {}
  );
  for (const [, value] of config.fastifyComponents.responses) {
    const response = traverseObject(
      opts.openapiObject,
      {
        type: "response",
        path: value.path
      },
      value.referenceObject,
      config.registry
    );
    if (!response) {
      throw new Error(
        `Response not found in OpenAPI object: ${value.path.join(" > ")}`
      );
    }
  }
  for (const [, value] of config.fastifyComponents.requestBodies) {
    const requestBody = traverseObject(
      opts.openapiObject,
      {
        type: "requestBody",
        path: value.path
      },
      value.referenceObject,
      config.registry
    );
    if (!requestBody) {
      throw new Error(
        `Request body not found in OpenAPI object: ${value.path.join(" > ")}`
      );
    }
  }
  for (const [, value] of config.registry.components.schemas.input) {
    const schema = traverseObject(
      opts.openapiObject,
      value.source,
      value.schemaObject,
      config.registry
    );
    if (!schema) {
      throw new Error(
        `Schema not found in OpenAPI object: ${value.source.path.join(" > ")}`
      );
    }
  }
  for (const [, value] of config.registry.components.schemas.output) {
    const schema = traverseObject(
      opts.openapiObject,
      value.source,
      value.schemaObject,
      config.registry
    );
    if (!schema) {
      throw new Error(
        `Schema not found in OpenAPI object: ${value.source.path.join(" > ")}`
      );
    }
  }
  return {
    ...opts.openapiObject,
    components: combineComponents(
      opts.openapiObject.components,
      components
    )
  };
};
const fastifyZodOpenApiTransformers = {
  transform: fastifyZodOpenApiTransform,
  transformObject: fastifyZodOpenApiTransformObject
};
const validatorCompiler = ({ schema }) => (value) => {
  const result = schema.safeParse(value);
  if (!result.success) {
    return {
      error: result.error.issues.map(
        (issue) => new RequestValidationError(
          issue.code,
          `/${issue.path.join("/")}`,
          `#/${issue.path.join("/")}/${issue.code}`,
          issue.message,
          {
            issue,
            error: result.error
          }
        )
      )
      // Types are wrong https://github.com/fastify/fastify/pull/5787
    };
  }
  return { value: result.data };
};
exports.FASTIFY_ZOD_OPENAPI_CONFIG = FASTIFY_ZOD_OPENAPI_CONFIG;
exports.RequestValidationError = RequestValidationError;
exports.ResponseSerializationError = ResponseSerializationError;
exports.createSerializerCompiler = createSerializerCompiler;
exports.fastifyZodOpenApiPlugin = fastifyZodOpenApiPlugin;
exports.fastifyZodOpenApiTransform = fastifyZodOpenApiTransform;
exports.fastifyZodOpenApiTransformObject = fastifyZodOpenApiTransformObject;
exports.fastifyZodOpenApiTransformers = fastifyZodOpenApiTransformers;
exports.serializerCompiler = serializerCompiler;
exports.validatorCompiler = validatorCompiler;
